@@ -3,7 +3,10 @@
 #ifndef NO_GENERICS
 {-# LANGUAGE DefaultSignatures, FlexibleContexts, TypeOperators #-}
 {-# LANGUAGE FlexibleInstances, KindSignatures, ScopedTypeVariables #-}
-{-# LANGUAGE MultiParamTypeClasses, OverlappingInstances #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
+#if __GLASGOW_HASKELL__ < 710
+{-# LANGUAGE OverlappingInstances  #-}
+#endif
 #endif
 #ifndef NO_SAFE_HASKELL
 {-# LANGUAGE Safe #-}
@@ -52,6 +55,7 @@ module Test.QuickCheck.Arbitrary
 -- imports
 
 import Control.Applicative
+import Data.Foldable(toList)
 import System.Random(Random)
 import Test.QuickCheck.Gen
 import Test.QuickCheck.Gen.Unsafe
@@ -115,6 +119,12 @@ import Data.Word(Word, Word8, Word16, Word32, Word64)
 import GHC.Generics
 #endif
 
+import qualified Data.Set as Set
+import qualified Data.Map as Map
+import qualified Data.IntSet as IntSet
+import qualified Data.IntMap as IntMap
+import qualified Data.Sequence as Sequence
+
 --------------------------------------------------------------------------
 -- ** class Arbitrary
 
@@ -122,7 +132,6 @@ import GHC.Generics
 class Arbitrary a where
   -- | A generator for values of the given type.
   arbitrary :: Gen a
-  arbitrary = error "no default generator"
 
   -- | Produces a (possibly) empty list of all the possible
   -- immediate shrinks of the given value. The default implementation
@@ -195,7 +204,7 @@ class Arbitrary a where
 #ifndef NO_GENERICS
 -- | Shrink a term to any of its immediate subterms,
 -- and also recursively shrink all subterms.
-genericShrink :: (Generic a, Arbitrary a, RecursivelyShrink (Rep a), GSubterms (Rep a) a) => a -> [a]
+genericShrink :: (Generic a, RecursivelyShrink (Rep a), GSubterms (Rep a) a) => a -> [a]
 genericShrink x = subterms x ++ recursivelyShrink x
 
 -- | Recursively shrink all immediate subterms.
@@ -229,7 +238,7 @@ instance RecursivelyShrink V1 where
 
 
 -- | All immediate subterms of a term.
-subterms :: (Generic a, Arbitrary a, GSubterms (Rep a) a) => a -> [a]
+subterms :: (Generic a, GSubterms (Rep a) a) => a -> [a]
 subterms = gSubterms . from
 
 
@@ -292,10 +301,10 @@ instance GSubtermsIncl f a => GSubtermsIncl (M1 i c f) a where
   gSubtermsIncl (M1 x) = gSubtermsIncl x
 
 -- This is the important case: We've found a term of the same type.
-instance Arbitrary a => GSubtermsIncl (K1 i a) a where
+instance {-# OVERLAPPING #-} GSubtermsIncl (K1 i a) a where
   gSubtermsIncl (K1 x) = [x]
 
-instance GSubtermsIncl (K1 i a) b where
+instance {-# OVERLAPPING #-} GSubtermsIncl (K1 i a) b where
   gSubtermsIncl (K1 _) = []
 
 #endif
@@ -365,7 +374,7 @@ shrinkList shr xs = concat [ removes k n xs | k <- takeWhile (>0) (iterate (`div
                ++ [ x':xs | x'  <- shrink x ]
 -}
 
-instance (Integral a, Arbitrary a) => Arbitrary (Ratio a) where
+instance Integral a => Arbitrary (Ratio a) where
   arbitrary = arbitrarySizedFractional
   shrink    = shrinkRealFracToInteger
 
@@ -494,6 +503,23 @@ instance Arbitrary Double where
   arbitrary = arbitrarySizedFractional
   shrink    = shrinkRealFrac
 
+-- Arbitrary instances for container types
+instance (Ord a, Arbitrary a) => Arbitrary (Set.Set a) where
+  arbitrary = fmap Set.fromList arbitrary
+  shrink = map Set.fromList . shrink . Set.toList
+instance (Ord k, Arbitrary k, Arbitrary v) => Arbitrary (Map.Map k v) where
+  arbitrary = fmap Map.fromList arbitrary
+  shrink = map Map.fromList . shrink . Map.toList
+instance Arbitrary IntSet.IntSet where
+  arbitrary = fmap IntSet.fromList arbitrary
+  shrink = map IntSet.fromList . shrink . IntSet.toList
+instance Arbitrary a => Arbitrary (IntMap.IntMap a) where
+  arbitrary = fmap IntMap.fromList arbitrary
+  shrink = map IntMap.fromList . shrink . IntMap.toList
+instance Arbitrary a => Arbitrary (Sequence.Seq a) where
+  arbitrary = fmap Sequence.fromList arbitrary
+  shrink = map Sequence.fromList . shrink . toList
+
 -- ** Helper functions for implementing arbitrary
 
 -- | Generates an integral number. The number can be positive or negative
@@ -799,6 +825,18 @@ instance CoArbitrary Float where
 instance CoArbitrary Double where
   coarbitrary = coarbitraryReal
 
+-- Coarbitrary instances for container types
+instance CoArbitrary a => CoArbitrary (Set.Set a) where
+  coarbitrary = coarbitrary. Set.toList
+instance (CoArbitrary k, CoArbitrary v) => CoArbitrary (Map.Map k v) where
+  coarbitrary = coarbitrary . Map.toList
+instance CoArbitrary IntSet.IntSet where
+  coarbitrary = coarbitrary . IntSet.toList
+instance CoArbitrary a => CoArbitrary (IntMap.IntMap a) where
+  coarbitrary = coarbitrary . IntMap.toList
+instance CoArbitrary a => CoArbitrary (Sequence.Seq a) where
+  coarbitrary = coarbitrary . toList
+
 -- ** Helpers for implementing coarbitrary
 
 -- | A 'coarbitrary' implementation for integral numbers.
@@ -826,7 +864,7 @@ coarbitraryEnum = variant . fromEnum
 vector :: Arbitrary a => Int -> Gen [a]
 vector k = vectorOf k arbitrary
 
--- | Generates an ordered list of a given length.
+-- | Generates an ordered list.
 orderedList :: (Ord a, Arbitrary a) => Gen [a]
 orderedList = sort `fmap` arbitrary
 
